<script setup lang="ts">

import dayjs from 'dayjs'
import {ScreenApi, statusMap, CallingVO} from "@/api/ecg/screen";

defineOptions({ name: 'bigscreen' })

const listReady = ref<CallingVO[]>([]) // 列表的数据
const listWaiting = ref<CallingVO[]>([]) // 列表的数据
const listPassed = ref<CallingVO[]>([]) //

let curSpeakPat : CallingVO | undefined = undefined;
let curSpeakSeqNum : number = 0
let curWorkDay : Date | undefined = undefined

const getList = async () => {
  const data = await ScreenApi.getBigScreenData()
  listReady.value = data[1]
  listWaiting.value = data[2]
  listPassed.value = data[3]
}

const startScrolling = () => {
  setInterval(() => {
    getList();
    if ( curSpeakPat === undefined ) {
      initiateSpeak()
    }
  }, 5000); // 每X秒滚动一次
}

onMounted( () => {
  startScrolling()
})

const onSpeachEndEvent = async (event) => {
  console.log("Speech ended" + event);
  await ScreenApi.markCalled(curSpeakPat!.patId)
  initiateSpeak()
}

const getNextSpeak = () => {
  if (listReady.value.length === 0) {
    return undefined;
  }

  // TODO 处理每天 反转的情况
  if ( curWorkDay === undefined) {
    curSpeakSeqNum = 0;
    curWorkDay = new Date()
  } else if (!isSameDate(dayjs(listReady.value[0].bookDate).toDate(), curWorkDay)) {
    curSpeakSeqNum = 0;
    curWorkDay = dayjs(listReady.value[0].bookDate).toDate()
  }

  return listReady.value.find( item => item.status === 30 && item.called === 0 && item.seqNum > curSpeakSeqNum)
}

const initiateSpeak = () => {
  const queuePat = getNextSpeak()
  if (queuePat !== undefined) {
    curSpeakPat = queuePat
    curSpeakSeqNum = curSpeakPat.seqNum
    speak(queuePat.patName + queuePat.roomName);
  } else {
    curSpeakPat = undefined
  }
}

const speak = (msg) => {
  var speech = new SpeechSynthesisUtterance()
  speech.text = msg
  speech.pitch = 1 // 获取并设置话语的音调(0-2 默认1，值越大越尖锐,越低越低沉)
  speech.rate = 0.9 // 获取并设置说话的速度(0.1-10 默认1，值越大语速越快,越小语速越慢)
  speech.volume = 100 // 获取并设置说话的音量
  speech.lang = 'zh-CN' // 设置播放语言

  speech.onend = onSpeachEndEvent
  speechSynthesis.speak(speech)
}

const isSameDate = ( date1: Date, date2 : Date) => {
  if (date1.getFullYear() === date2.getFullYear()
      && date1.getMonth() === date2.getMonth()
      && date1.getDate() === date2.getDate())
    return true
  else
    return false
}

</script>

<template>
  <el-container>
    <el-header>Header</el-header>
    <el-container>
      <el-aside width="500px">
        <el-table
            :data="listReady"
            stripe
            :show-header="false"
            style="width: 100%">
          <el-table-column
              prop="patName"
              label="患者姓名"
              width="80"/>
          <el-table-column label="状态" align="center" prop="status" width="80">
            <template #default="scope">
              {{ statusMap.get(scope.row.status) }}
            </template>
          </el-table-column>
        </el-table>
      </el-aside>
      <el-container>
        <el-main>
          <el-table
            :data="listWaiting"
            stripe
            :show-header="false"
            style="width: 100%">
            <el-table-column
              prop="patName"
              label="患者姓名"
              width="80"/>
            <el-table-column label="状态" align="center" prop="status" width="80">
              <template #default="scope">
                {{ statusMap.get(scope.row.status) }}
              </template>
            </el-table-column>
          </el-table>
        </el-main>
        <el-footer height="100px">
          <el-table
            :data="listPassed"
            stripe
            :show-header="false"
            style="width: 100%">
            <el-table-column
              prop="patName"
              label="患者姓名"
              width="80"/>
            <el-table-column label="状态" align="center" prop="status" width="80">
              <template #default="scope">
                {{ statusMap.get(scope.row.status) }}
              </template>
            </el-table-column>
          </el-table>
        </el-footer>
      </el-container>
    </el-container>
    <el-button @click="speak('请 特朗普 到二号诊室 就诊')" >叫号</el-button>
  </el-container>
</template>

<style scoped lang="scss">
.el-header, .el-footer {
  background-color: #B3C0D1;
  color: #333;
  text-align: center;
  line-height: 60px;
}

.el-aside {
  background-color: #D3DCE6;
  color: #333;
  text-align: center;
  line-height: 200px;
}

.el-main {
  background-color: #E9EEF3;
  color: #333;
  text-align: center;
  line-height: 160px;
}

body > .el-container {
  margin-bottom: 40px;
}

.el-container:nth-child(5) .el-aside,
.el-container:nth-child(6) .el-aside {
  line-height: 260px;
}

.el-container:nth-child(7) .el-aside {
  line-height: 320px;
}
</style>
